package dev.fitko.fitconnect.core.destination;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.auth.OAuthToken;
import dev.fitko.fitconnect.api.domain.model.destination.Destination;
import dev.fitko.fitconnect.api.domain.model.jwk.ApiJwk;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.services.auth.OAuthService;
import dev.fitko.fitconnect.api.services.destination.DestinationService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.UUID;

public class DestinationApiService implements DestinationService {

    private final OAuthService authService;
    private final RestTemplate restTemplate;
    private final ApplicationConfig config;

    public DestinationApiService(final OAuthService authService, final RestTemplate restTemplate, final ApplicationConfig config) {
        this.authService = authService;
        this.restTemplate = restTemplate;
        this.config = config;
    }

    @Override
    public Destination getDestination(final UUID destinationID) {
        final String url = this.config.getDestinationsEndpoint();
        final OAuthToken token = this.authService.getCurrentToken();
        return performRequest(token.getAccessToken(), url, Destination.class, destinationID);
    }

    @Override
    public ApiJwk getPublicKey(final UUID destinationID, final String keyId) {
        final String url = this.config.getDestinationsKeyEndpoint();
        final OAuthToken token = this.authService.getCurrentToken();
        return performRequest(token.getAccessToken(), url, ApiJwk.class, destinationID, keyId);
    }

    private <T> T performRequest(final String accessToken, final String url, final Class<T> responseType, final Object... params) {
        final HttpHeaders headers = getHeaders(accessToken);
        final HttpEntity<String> entity = new HttpEntity<>(headers);
        try {
            return this.restTemplate.exchange(url, HttpMethod.GET, entity, responseType, params).getBody();
        } catch (final RestClientException e) {
            throw new RestApiException("Request failed", e);
        }
    }

    private HttpHeaders getHeaders(final String oAuthToken) {
        final var headers = new HttpHeaders();
        headers.setBearerAuth(oAuthToken);
        return headers;
    }
}
